function outKernel = Compute_Grassmann_Kernel(SY1,SY2)
%%
% Kernel computation for Grassmannian Graph-Embedding Discriminant Analysis
%
% OUTKERNEL = Compute_Grassmann_Kernel(SY1,SY2)
%       Input:
%           SY1 - Orthonormalized subspaces (cell) of gallery.
%           SY2 - Orthonormalized subspaces (cell) of probe.
%       Output:
%           outKernel - Grassmann kernel matrix for the inputs. outKernel is a 3D 
%                       matrix, if only SY1 is provided, then outKernel(i,j,1) is 
%                       the projection kernel between SY1(:,:,i) and SY1(:,:,j). 
%                       Similarly, outKernel(i,j,2) is the CC kernel(equ.14 of the
%                       reference) between SY1(:,:,i) and SY1(:,:,j). if SY2 is 
%                       provided, then outKernel(i,j,1) is the projection kernel 
%                       between SY1(:,:,i) and SY2(:,:,j). Similarly, outKernel(i,j,2)
%                       is the CC kernel(equ.14 of the reference) between SY1(:,:,i) 
%                       and SY2(:,:,j).
%                       
% Reference:
%
%   M.T. Harandi, C. Sanderson, S. Shirazi, and B.C. Lovell, 
%   Graph Embedding Discriminant Analysis on Grassmannian Manifold for 
%   Improved Set Matching, CVPR'2011.
%
%   Edited and revised by Mengyi Liu (mengyi.liu@vipl.ict.ac.cn)
%
%%
if (nargin<1)
    error('Not enough inputs');
end

number_sets1=length(SY1);

if (isempty(SY2)~=1)
    number_sets2 = length(SY2);
else
    SY2 = SY1;
    number_sets2 = length(SY2);
end

outKernel = zeros(number_sets1,number_sets2,1);
for tmpC1 = 1:number_sets1
    Y1 = SY1{tmpC1};
    for tmpC2 = 1:number_sets2
        Y2 = SY2{tmpC2};
        if(isempty(Y1)~=1 && isempty(Y2)~=1)
            %Finding the minimum rank
            Min_Rank = min(rank(Y1),rank(Y2));
            tmpMatrix = Y1(:,1:Min_Rank)'*Y2(:,1:Min_Rank);
            
            %Projection kernel
            outKernel(tmpC1,tmpC2,1) = sum(sum(tmpMatrix.^2));
                     
            %Canonical correlation
            [~,D,~] = svd(tmpMatrix);
            outKernel(tmpC1,tmpC2,2) = D(1);
        else
            outKernel(tmpC1,tmpC2,1) = 0;
            outKernel(tmpC1,tmpC2,2) = 0;
        end
    end
%     fprintf('GGDA data: %d\n', tmpC1);
end